/* src/bazaar.p/bz-exponential-function.h
 *
 * Copyright 2026 AUTOGEN
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 */

#pragma once

#include <glib-object.h>


G_BEGIN_DECLS



#define BZ_TYPE_EXPONENTIAL_FUNCTION (bz_exponential_function_get_type ())
G_DECLARE_FINAL_TYPE (BzExponentialFunction, bz_exponential_function, BZ, EXPONENTIAL_FUNCTION, GObject)

BzExponentialFunction *
bz_exponential_function_new (void);

double
bz_exponential_function_get_factor (BzExponentialFunction *self);

double
bz_exponential_function_get_y_intercept (BzExponentialFunction *self);

void
bz_exponential_function_set_factor (BzExponentialFunction *self,
    double factor);

void
bz_exponential_function_set_y_intercept (BzExponentialFunction *self,
    double y_intercept);



G_END_DECLS

/* End of src/bazaar.p/bz-exponential-function.h */
