/* src/bazaar.p/bz-transaction-dialog-result.h
 *
 * Copyright 2026 AUTOGEN
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 */

#pragma once

#include <glib-object.h>
#include "bz-entry.h"

G_BEGIN_DECLS



#define BZ_TYPE_TRANSACTION_DIALOG_RESULT (bz_transaction_dialog_result_get_type ())
G_DECLARE_FINAL_TYPE (BzTransactionDialogResult, bz_transaction_dialog_result, BZ, TRANSACTION_DIALOG_RESULT, GObject)

BzTransactionDialogResult *
bz_transaction_dialog_result_new (void);

BzEntry *
bz_transaction_dialog_result_get_selected_entry (BzTransactionDialogResult *self);

gboolean
bz_transaction_dialog_result_get_delete_user_data (BzTransactionDialogResult *self);

gboolean
bz_transaction_dialog_result_get_confirmed (BzTransactionDialogResult *self);

void
bz_transaction_dialog_result_set_selected_entry (BzTransactionDialogResult *self,
    BzEntry *selected_entry);

void
bz_transaction_dialog_result_set_delete_user_data (BzTransactionDialogResult *self,
    gboolean delete_user_data);

void
bz_transaction_dialog_result_set_confirmed (BzTransactionDialogResult *self,
    gboolean confirmed);



G_END_DECLS

/* End of src/bazaar.p/bz-transaction-dialog-result.h */
