/* This file is generated by glib-genmarshal, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */
#pragma once

#include <glib-object.h>

G_BEGIN_DECLS

/* OBJECT:OBJECT (../bge/bge-marshalers.list:28) */
extern
void bge_marshal_OBJECT__OBJECT (GClosure     *closure,
                                 GValue       *return_value,
                                 guint         n_param_values,
                                 const GValue *param_values,
                                 gpointer      invocation_hint,
                                 gpointer      marshal_data);
extern
void bge_marshal_OBJECT__OBJECTv (GClosure *closure,
                                  GValue   *return_value,
                                  gpointer  instance,
                                  va_list   args,
                                  gpointer  marshal_data,
                                  int       n_params,
                                  GType    *param_types);

/* VOID:OBJECT,OBJECT (../bge/bge-marshalers.list:29) */
extern
void bge_marshal_VOID__OBJECT_OBJECT (GClosure     *closure,
                                      GValue       *return_value,
                                      guint         n_param_values,
                                      const GValue *param_values,
                                      gpointer      invocation_hint,
                                      gpointer      marshal_data);
extern
void bge_marshal_VOID__OBJECT_OBJECTv (GClosure *closure,
                                       GValue   *return_value,
                                       gpointer  instance,
                                       va_list   args,
                                       gpointer  marshal_data,
                                       int       n_params,
                                       GType    *param_types);

/* DOUBLE:DOUBLE (../bge/bge-marshalers.list:34) */
extern
void bge_marshal_DOUBLE__DOUBLE (GClosure     *closure,
                                 GValue       *return_value,
                                 guint         n_param_values,
                                 const GValue *param_values,
                                 gpointer      invocation_hint,
                                 gpointer      marshal_data);
extern
void bge_marshal_DOUBLE__DOUBLEv (GClosure *closure,
                                  GValue   *return_value,
                                  gpointer  instance,
                                  va_list   args,
                                  gpointer  marshal_data,
                                  int       n_params,
                                  GType    *param_types);

/* DOUBLE:DOUBLE,DOUBLE (../bge/bge-marshalers.list:35) */
extern
void bge_marshal_DOUBLE__DOUBLE_DOUBLE (GClosure     *closure,
                                        GValue       *return_value,
                                        guint         n_param_values,
                                        const GValue *param_values,
                                        gpointer      invocation_hint,
                                        gpointer      marshal_data);
extern
void bge_marshal_DOUBLE__DOUBLE_DOUBLEv (GClosure *closure,
                                         GValue   *return_value,
                                         gpointer  instance,
                                         va_list   args,
                                         gpointer  marshal_data,
                                         int       n_params,
                                         GType    *param_types);

/* DOUBLE:DOUBLE,DOUBLE,DOUBLE (../bge/bge-marshalers.list:36) */
extern
void bge_marshal_DOUBLE__DOUBLE_DOUBLE_DOUBLE (GClosure     *closure,
                                               GValue       *return_value,
                                               guint         n_param_values,
                                               const GValue *param_values,
                                               gpointer      invocation_hint,
                                               gpointer      marshal_data);
extern
void bge_marshal_DOUBLE__DOUBLE_DOUBLE_DOUBLEv (GClosure *closure,
                                                GValue   *return_value,
                                                gpointer  instance,
                                                va_list   args,
                                                gpointer  marshal_data,
                                                int       n_params,
                                                GType    *param_types);

/* DOUBLE:DOUBLE,DOUBLE,DOUBLE,DOUBLE (../bge/bge-marshalers.list:37) */
extern
void bge_marshal_DOUBLE__DOUBLE_DOUBLE_DOUBLE_DOUBLE (GClosure     *closure,
                                                      GValue       *return_value,
                                                      guint         n_param_values,
                                                      const GValue *param_values,
                                                      gpointer      invocation_hint,
                                                      gpointer      marshal_data);
extern
void bge_marshal_DOUBLE__DOUBLE_DOUBLE_DOUBLE_DOUBLEv (GClosure *closure,
                                                       GValue   *return_value,
                                                       gpointer  instance,
                                                       va_list   args,
                                                       gpointer  marshal_data,
                                                       int       n_params,
                                                       GType    *param_types);

/* DOUBLE:DOUBLE,DOUBLE,DOUBLE,DOUBLE,DOUBLE (../bge/bge-marshalers.list:38) */
extern
void bge_marshal_DOUBLE__DOUBLE_DOUBLE_DOUBLE_DOUBLE_DOUBLE (GClosure     *closure,
                                                             GValue       *return_value,
                                                             guint         n_param_values,
                                                             const GValue *param_values,
                                                             gpointer      invocation_hint,
                                                             gpointer      marshal_data);
extern
void bge_marshal_DOUBLE__DOUBLE_DOUBLE_DOUBLE_DOUBLE_DOUBLEv (GClosure *closure,
                                                              GValue   *return_value,
                                                              gpointer  instance,
                                                              va_list   args,
                                                              gpointer  marshal_data,
                                                              int       n_params,
                                                              GType    *param_types);

/* BOXED:DOUBLE,DOUBLE (../bge/bge-marshalers.list:39) */
extern
void bge_marshal_BOXED__DOUBLE_DOUBLE (GClosure     *closure,
                                       GValue       *return_value,
                                       guint         n_param_values,
                                       const GValue *param_values,
                                       gpointer      invocation_hint,
                                       gpointer      marshal_data);
extern
void bge_marshal_BOXED__DOUBLE_DOUBLEv (GClosure *closure,
                                        GValue   *return_value,
                                        gpointer  instance,
                                        va_list   args,
                                        gpointer  marshal_data,
                                        int       n_params,
                                        GType    *param_types);

/* BOXED:DOUBLE,DOUBLE,DOUBLE (../bge/bge-marshalers.list:40) */
extern
void bge_marshal_BOXED__DOUBLE_DOUBLE_DOUBLE (GClosure     *closure,
                                              GValue       *return_value,
                                              guint         n_param_values,
                                              const GValue *param_values,
                                              gpointer      invocation_hint,
                                              gpointer      marshal_data);
extern
void bge_marshal_BOXED__DOUBLE_DOUBLE_DOUBLEv (GClosure *closure,
                                               GValue   *return_value,
                                               gpointer  instance,
                                               va_list   args,
                                               gpointer  marshal_data,
                                               int       n_params,
                                               GType    *param_types);

/* BOXED:DOUBLE,DOUBLE,DOUBLE,DOUBLE (../bge/bge-marshalers.list:41) */
extern
void bge_marshal_BOXED__DOUBLE_DOUBLE_DOUBLE_DOUBLE (GClosure     *closure,
                                                     GValue       *return_value,
                                                     guint         n_param_values,
                                                     const GValue *param_values,
                                                     gpointer      invocation_hint,
                                                     gpointer      marshal_data);
extern
void bge_marshal_BOXED__DOUBLE_DOUBLE_DOUBLE_DOUBLEv (GClosure *closure,
                                                      GValue   *return_value,
                                                      gpointer  instance,
                                                      va_list   args,
                                                      gpointer  marshal_data,
                                                      int       n_params,
                                                      GType    *param_types);

/* INT:DOUBLE (../bge/bge-marshalers.list:43) */
extern
void bge_marshal_INT__DOUBLE (GClosure     *closure,
                              GValue       *return_value,
                              guint         n_param_values,
                              const GValue *param_values,
                              gpointer      invocation_hint,
                              gpointer      marshal_data);
extern
void bge_marshal_INT__DOUBLEv (GClosure *closure,
                               GValue   *return_value,
                               gpointer  instance,
                               va_list   args,
                               gpointer  marshal_data,
                               int       n_params,
                               GType    *param_types);

/* OBJECT:DOUBLE (../bge/bge-marshalers.list:44) */
extern
void bge_marshal_OBJECT__DOUBLE (GClosure     *closure,
                                 GValue       *return_value,
                                 guint         n_param_values,
                                 const GValue *param_values,
                                 gpointer      invocation_hint,
                                 gpointer      marshal_data);
extern
void bge_marshal_OBJECT__DOUBLEv (GClosure *closure,
                                  GValue   *return_value,
                                  gpointer  instance,
                                  va_list   args,
                                  gpointer  marshal_data,
                                  int       n_params,
                                  GType    *param_types);


G_END_DECLS
