/* src/bazaar.p/bz-flathub-auth-provider.h
 *
 * Copyright 2026 AUTOGEN
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 */

#pragma once

#include <glib-object.h>


G_BEGIN_DECLS



#define BZ_TYPE_FLATHUB_AUTH_PROVIDER (bz_flathub_auth_provider_get_type ())
G_DECLARE_FINAL_TYPE (BzFlathubAuthProvider, bz_flathub_auth_provider, BZ, FLATHUB_AUTH_PROVIDER, GObject)

BzFlathubAuthProvider *
bz_flathub_auth_provider_new (void);

const char *
bz_flathub_auth_provider_get_method (BzFlathubAuthProvider *self);

const char *
bz_flathub_auth_provider_get_name (BzFlathubAuthProvider *self);

void
bz_flathub_auth_provider_set_method (BzFlathubAuthProvider *self,
    const char *method);

void
bz_flathub_auth_provider_set_name (BzFlathubAuthProvider *self,
    const char *name);

void
bz_flathub_auth_provider_set_method_take (BzFlathubAuthProvider *self,
    char *method);

#define bz_flathub_auth_provider_set_method_take_printf(self, ...) bz_flathub_auth_provider_set_method_take (self, g_strdup_printf (__VA_ARGS__))

void
bz_flathub_auth_provider_set_name_take (BzFlathubAuthProvider *self,
    char *name);

#define bz_flathub_auth_provider_set_name_take_printf(self, ...) bz_flathub_auth_provider_set_name_take (self, g_strdup_printf (__VA_ARGS__))

G_END_DECLS

/* End of src/bazaar.p/bz-flathub-auth-provider.h */
