/* src/bazaar.p/bz-finished-search-query.h
 *
 * Copyright 2026 AUTOGEN
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 */

#pragma once

#include <glib-object.h>


G_BEGIN_DECLS



#define BZ_TYPE_FINISHED_SEARCH_QUERY (bz_finished_search_query_get_type ())
G_DECLARE_FINAL_TYPE (BzFinishedSearchQuery, bz_finished_search_query, BZ, FINISHED_SEARCH_QUERY, GObject)

BzFinishedSearchQuery *
bz_finished_search_query_new (void);

const char *
bz_finished_search_query_get_interpreted_query (BzFinishedSearchQuery *self);

GPtrArray *
bz_finished_search_query_get_results (BzFinishedSearchQuery *self);

guint
bz_finished_search_query_get_n_results (BzFinishedSearchQuery *self);

double
bz_finished_search_query_get_elapsed (BzFinishedSearchQuery *self);

void
bz_finished_search_query_set_interpreted_query (BzFinishedSearchQuery *self,
    const char *interpreted_query);

void
bz_finished_search_query_set_results (BzFinishedSearchQuery *self,
    GPtrArray *results);

void
bz_finished_search_query_set_n_results (BzFinishedSearchQuery *self,
    guint n_results);

void
bz_finished_search_query_set_elapsed (BzFinishedSearchQuery *self,
    double elapsed);

void
bz_finished_search_query_set_interpreted_query_take (BzFinishedSearchQuery *self,
    char *interpreted_query);

#define bz_finished_search_query_set_interpreted_query_take_printf(self, ...) bz_finished_search_query_set_interpreted_query_take (self, g_strdup_printf (__VA_ARGS__))

G_END_DECLS

/* End of src/bazaar.p/bz-finished-search-query.h */
