/* src/bazaar.p/bz-pride-flag-config.h
 *
 * Copyright 2026 AUTOGEN
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 */

#pragma once

#include <glib-object.h>
#include <gtk/gtk.h>
#include "bz-pride-flag-spec.h"

G_BEGIN_DECLS



#define BZ_TYPE_PRIDE_FLAG_CONFIG (bz_pride_flag_config_get_type ())
G_DECLARE_FINAL_TYPE (BzPrideFlagConfig, bz_pride_flag_config, BZ, PRIDE_FLAG_CONFIG, GObject)

BzPrideFlagConfig *
bz_pride_flag_config_new (void);

GListModel *
bz_pride_flag_config_get_flag_specs (BzPrideFlagConfig *self);

void
bz_pride_flag_config_set_flag_specs (BzPrideFlagConfig *self,
    GListModel *flag_specs);



G_END_DECLS

/* End of src/bazaar.p/bz-pride-flag-config.h */
