/* bge-wdgt-spec.h
 *
 * Copyright 2026 Eva M
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 */

#pragma once

#ifndef BGE_INSIDE
#error "Only <bge.h> can be included directly."
#endif

G_BEGIN_DECLS

#define BGE_TYPE_WDGT_SPEC (bge_wdgt_spec_get_type ())
G_DECLARE_FINAL_TYPE (BgeWdgtSpec, bge_wdgt_spec, BGE, WDGT_SPEC, GObject)

BGE_AVAILABLE_IN_ALL
BgeWdgtSpec *
bge_wdgt_spec_new (void);

BGE_AVAILABLE_IN_ALL
BgeWdgtSpec *
bge_wdgt_spec_new_for_string (const char *string,
                              GError    **error);

BGE_AVAILABLE_IN_ALL
BgeWdgtSpec *
bge_wdgt_spec_new_for_resource (const char *resource,
                                GError    **error);

BGE_AVAILABLE_IN_ALL
const char *
bge_wdgt_spec_get_name (BgeWdgtSpec *self);

BGE_AVAILABLE_IN_ALL
void
bge_wdgt_spec_set_name (BgeWdgtSpec *self,
                        const char  *name);

BGE_AVAILABLE_IN_ALL
void
bge_wdgt_spec_set_name_take (BgeWdgtSpec *self,
                             char        *name);

#define bge_wdgt_spec_set_name_take_printf(self, ...) bge_wdgt_spec_set_name_take (self, g_strdup_printf (__VA_ARGS__))

G_END_DECLS

/* End of bge-wdgt-spec.h */
